import { AfterViewInit, ChangeDetectorRef, EventEmitter, Injector, OnDestroy, OnInit } from '@angular/core';
import { IonRouterOutlet } from '@ionic/angular';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { AddToPageHistoryOptions, LocalSettingsService } from '../../services/local-settings.service';
import { Entity } from '../../services/model/entity.model';
import { UsageMode } from '../../services/model/settings.model';
import { FormGroup } from '@angular/forms';
import { AppTabEditor, AppTabEditorOptions, IAppTabEditor } from './tab-editor.class';
import { AppErrorWithDetails } from '../../services/errors';
import { EntityServiceLoadOptions, IEntityService } from '../../../shared/services/entity-service.class';
import { DateFormatService } from '../../../shared/pipes/date-format.pipe';
import { Environment } from '../../../../environments/environment.class';
import { HistoryPageReference } from '../../services/model/history.model';
import { WaitForOptions } from '../../../shared/observables';
import { FormErrorTranslator } from '../../../shared/validator/form-error-adapter.class';
import * as i0 from "@angular/core";
export interface IAppEntityEditor<T> extends IAppTabEditor<T> {
    usageMode: UsageMode;
    isNewData: boolean;
}
export declare class AppEditorOptions extends AppTabEditorOptions {
    autoLoad?: boolean;
    autoLoadDelay?: number;
    pathIdAttribute?: string;
    enableListenChanges?: boolean;
    listenIntervalInSeconds?: number;
    /**
     * Change page route (window URL) when saving for the first time
     */
    autoUpdateRoute?: boolean;
    /**
     * Open the next tab, after saving for the first time
     */
    autoOpenNextTab?: boolean;
    i18nPrefix?: string;
}
export declare abstract class AppEntityEditor<T extends Entity<T, ID>, S extends IEntityService<T, ID> = IEntityService<T, any>, ID = number, LO extends EntityServiceLoadOptions = EntityServiceLoadOptions> extends AppTabEditor<T, ID, LO> implements OnInit, OnDestroy, AfterViewInit, IAppEntityEditor<T> {
    protected dataType: new () => T;
    protected dataService?: S;
    data: T;
    previousDataId: ID;
    defaultBackHref: string;
    historyIcon: {
        icon?: string;
        matIcon?: string;
    };
    readonly titleSubject: BehaviorSubject<string>;
    readonly onUpdateView: EventEmitter<T>;
    /**
     *  @deprecated
     */
    protected get $title(): Subject<string>;
    protected readonly cd: ChangeDetectorRef;
    protected readonly dateFormat: DateFormatService;
    protected readonly settings: LocalSettingsService;
    protected readonly environment: Environment;
    protected readonly routerOutlet: IonRouterOutlet;
    protected errorTranslator: FormErrorTranslator;
    private _usageMode;
    private readonly _enableListenChanges;
    private readonly _listenIntervalInSeconds;
    private readonly _pathIdAttribute;
    private readonly _autoLoad;
    private readonly _autoLoadDelay;
    private readonly _autoUpdateRoute;
    private _autoOpenNextTab;
    private _listenChangesSubscription;
    get usageMode(): UsageMode;
    set usageMode(value: UsageMode);
    get isOnFieldMode(): boolean;
    get isNewData(): boolean;
    get service(): S;
    get autoOpenNextTab(): boolean;
    set autoOpenNextTab(value: boolean);
    protected get pathIdAttribute(): string;
    protected constructor(injector: Injector, dataType: new () => T, dataService?: S, options?: AppEditorOptions);
    ngOnInit(): void;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    waitIdle(opts?: WaitForOptions): Promise<void>;
    /**
     * Load data from id, using the dataService
     *
     * @param id
     * @param opts
     */
    load(id?: ID, opts?: EntityServiceLoadOptions & {
        emitEvent?: boolean;
        openTabIndex?: number;
        updateRoute?: boolean;
        [key: string]: any;
    }): Promise<void>;
    canUserWrite(data: T, opts?: any): boolean;
    startListenChanges(): void;
    stopListenChanges(): void;
    ready(opts?: WaitForOptions): Promise<void>;
    updateView(data: T | null, opts?: {
        emitEvent?: boolean;
        openTabIndex?: number;
        updateRoute?: boolean;
    }): Promise<void>;
    /**
     * Enable or disable state
     */
    updateViewState(data: T, opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     * Update the route location, and open the next tab
     */
    updateTabAndRoute(data: T, opts?: {
        openTabIndex?: number;
    }): Promise<void>;
    /**
     * Update the route location, and open the next tab
     */
    updateTabIndex(tabIndex?: number): void;
    saveAndClose(event: Event, options?: any): Promise<boolean>;
    close(event: Event): Promise<boolean | void>;
    /**
     * Save the editor, by calling the dataService.save().
     * Ensure that editor if valid. If not, display an error
     *
     * @param event
     * @param opts
     */
    save(event?: Event, opts?: any): Promise<boolean>;
    /**
     * Save data (if dirty and valid), and return it. Otherwise, return nil value.
     */
    saveAndGetDataIfValid(): Promise<T | undefined>;
    delete(event?: Event): Promise<boolean>;
    reload(): Promise<void>;
    unload(opts?: {
        emitEvent?: boolean;
    }): Promise<void>;
    goBack(event?: Event): Promise<void | boolean>;
    resetError(opts?: {
        emitEvent?: boolean;
    }): void;
    setError(err: string | AppErrorWithDetails, opts?: {
        emitEvent?: boolean;
        detailsCssClass?: string;
    }): void;
    /**
     * Load data from the snapshot route
     *
     * @protected
     */
    protected loadFromRoute(): Promise<void>;
    /**
     * Return undefined to keep same tab
     *
     * @protected
     */
    protected computeNextTabIndex(): number | undefined;
    protected onBeforeEntityLoad(): Promise<void>;
    protected onNewEntity(data: T, options?: EntityServiceLoadOptions): Promise<void>;
    protected onEntityLoaded(data: T, options?: EntityServiceLoadOptions): Promise<void>;
    protected onEntitySaved(data: T): Promise<void>;
    protected onEntityDeleted(data: T): Promise<void>;
    protected computeUsageMode(data: T): UsageMode;
    protected waitWhilePending(opts?: WaitForOptions): Promise<void>;
    getValue(): Promise<T>;
    protected getJsonValueToSave(): Promise<any>;
    /**
     * Compute the title
     *
     * @param data
     */
    protected updateTitle(data?: T): Promise<void>;
    protected addToPageHistory(page: HistoryPageReference, opts?: AddToPageHistoryOptions): Promise<void>;
    protected computePageHistory(title: string): Promise<HistoryPageReference>;
    protected removePageHistory(opts?: {
        emitEvent?: boolean;
    }): Promise<any>;
    protected computePageUrl(id: ID | 'new'): string | any[];
    protected getParentPageUrl(withQueryParams?: boolean): string;
    protected listenChanges(id: ID, opts?: any): Observable<T | undefined>;
    protected markForCheck(): void;
    /**
     * Update the route, without reloading the component
     *
     * @param data
     * @param queryParams
     * @protected
     */
    protected updateRoute(data?: T, queryParams?: any): Promise<boolean>;
    /**
     * Open the first tab that is invalid
     */
    private openFirstInvalidTab;
    protected abstract computeTitle(data: T): Promise<string>;
    protected abstract get form(): FormGroup;
    protected abstract getFirstInvalidTabIndex(): number;
    protected abstract registerForms(): void;
    abstract setValue(data: T, opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): Promise<void> | void;
    static ɵfac: i0.ɵɵFactoryDeclaration<AppEntityEditor<any, any, any, any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<AppEntityEditor<any, any, any, any>, never, never, {}, {}, never, never, false, never>;
}
