import { AbstractControl } from '@angular/forms';
import { filterNumberInput, selectInputContent, selectInputContentFromEvent } from '../../shared/inputs';
import { WaitForOptions } from '../../shared/observables';
import { addValueInArray, clearValueInArray, copyEntity2Form, disableAndClearControl, disableAndClearControls, disableControl, disableControls, enableControl, enableControls, filterFormErrors, filterFormErrorsByPath, filterFormErrorsByPrefix, getControlFromPath, getFormErrors, getFormValueFromEntity, logFormErrors, markAllAsTouched, markAsUntouched, removeValueInArray, resizeArray, setControlEnabled, setControlsEnabled, setFormErrors, updateValueAndValidity, waitIdle, waitWhilePending } from '../../shared/forms';
export declare type IAppFormGetter = () => IAppForm;
export declare interface OnReady {
    ngOnReady(): any;
}
export interface IAppForm<T = any> {
    invalid: boolean;
    valid: boolean;
    dirty: boolean;
    pending: boolean;
    error: string;
    enabled: boolean;
    disabled: boolean;
    touched: boolean;
    untouched: boolean;
    disable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    enable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    empty?: boolean;
    loading?: boolean;
    loaded?: boolean;
    destroyed?: boolean;
    ready(): Promise<void>;
    waitIdle(opts?: WaitForOptions): Promise<any>;
    setValue?: (data: T, opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }) => Promise<void> | void;
    patchValue?: (data: T, opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }) => Promise<void> | void;
    reset?: (data?: T, opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }) => void;
    getValue?: () => Promise<T> | T;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAllAsTouched(opts?: {
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoading(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoaded(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsReady(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
}
export interface CanSave {
    dirty: boolean;
    valid: boolean;
    save(): Promise<boolean>;
}
export interface CanLeave extends CanSave {
    cancel(event?: Event): Promise<void>;
    deactivate?: () => Promise<void>;
}
/**
 * A form that do nothing
 */
export declare class AppNullForm implements IAppForm {
    readonly invalid = false;
    readonly valid = false;
    readonly dirty = false;
    readonly empty = true;
    readonly pending = false;
    readonly error: any;
    readonly enabled = false;
    readonly loading = false;
    readonly touched = false;
    get disabled(): boolean;
    get untouched(): boolean;
    disable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    enable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAllAsTouched(opts?: {
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoading(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoaded(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsReady(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    waitIdle(opts?: WaitForOptions): Promise<any>;
    ready(): Promise<any>;
}
export declare class AppFormProvider<F extends IAppForm = IAppForm> implements IAppForm {
    private getterFn;
    static NULL_FORM: AppNullForm;
    constructor(getterFn: () => F);
    private get delegate();
    waitDelegate(opts?: WaitForOptions): Promise<F>;
    get enabled(): boolean;
    get disabled(): boolean;
    get error(): string;
    get invalid(): boolean;
    get valid(): boolean;
    get dirty(): boolean;
    get empty(): boolean;
    get pending(): boolean;
    get loading(): boolean;
    get touched(): boolean;
    get untouched(): boolean;
    ready(): Promise<any>;
    waitIdle(opts?: WaitForOptions): Promise<any>;
    disable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    enable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAllAsTouched(opts?: {
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoading(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsLoaded(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsReady(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    save(): Promise<boolean>;
}
/**
 * Helper class, for angular forms
 */
export declare class AppFormUtils {
    static copyEntity2Form: typeof copyEntity2Form;
    static getFormValueFromEntity: typeof getFormValueFromEntity;
    /**
     * Retrieves a control from the provided form/control structure using a dot-separated path.
     *
     * @param {AbstractControl} form - The root form group, form array, or control to start searching from.
     * @param {string} path - The dot-separated path specifying the control. For example, 'groupName.controlName' or 'arrayName.0.controlName'.
     * @param {boolean} [strict=true] - If true, throws an error if the full path cannot be resolved; if false, returns the last valid control encountered.
     * @return {AbstractControl} - The control at the specified path or the closest valid control if strict mode is disabled.
     * @throws {Error} - Throws an error if the path cannot be resolved and strict mode is enabled.
     */
    static getControlFromPath: typeof getControlFromPath;
    static filterNumberInput: typeof filterNumberInput;
    static enableControls: typeof enableControls;
    static enableControl: typeof enableControl;
    static disableControl: typeof disableControl;
    static disableControls: typeof disableControls;
    static disableAndClearControl: typeof disableAndClearControl;
    static disableAndClearControls: typeof disableAndClearControls;
    static setControlEnabled: typeof setControlEnabled;
    static setControlsEnabled: typeof setControlsEnabled;
    static selectInputContentFromEvent: typeof selectInputContentFromEvent;
    static selectInputContent: typeof selectInputContent;
    /**
     * @deprecated use markAllAsTouched() instead (naming was wrong)
     */
    static markAsTouched: typeof markAllAsTouched;
    /**
     * Marks the control and all its descendant controls as touched.
     * See also:
     * - `markAsTouched()`
     */
    static markAllAsTouched: typeof markAllAsTouched;
    static markAsUntouched: typeof markAsUntouched;
    static waitWhilePending: typeof waitWhilePending;
    static waitIdle: typeof waitIdle;
    static updateValueAndValidity: typeof updateValueAndValidity;
    static isForm(object: any): object is IAppForm;
    static canSave(object: any): object is IAppForm & CanSave;
    static addValueInArray: typeof addValueInArray;
    static removeValueInArray: typeof removeValueInArray;
    static resizeArray: typeof resizeArray;
    static clearValueInArray: typeof clearValueInArray;
    static calculatedSuffix: string;
    static isControlHasInput: typeof isControlHasInput;
    static setCalculatedValue: typeof setCalculatedValue;
    static resetCalculatedValue: typeof resetCalculatedValue;
    static logFormErrors: typeof logFormErrors;
    static filterErrorsByPrefix: typeof filterFormErrorsByPrefix;
    static filterErrorsByPath: typeof filterFormErrorsByPath;
    static filterErrors: typeof filterFormErrors;
    /**
     * Retrieves all validation errors from a given form control, including its child controls if it is a form group or array.
     *
     * @param {AbstractControl} control - The form control to retrieve errors from. This can be a form group, a form array, or a simple form control.
     * @param {Object} [opts] - Optional configurations for error retrieval.
     * @param {string} [opts.controlName] - The name or path of the control being processed, used for structuring error keys in the result.
     * @param {FormErrors} [opts.result] - The object to which errors will be added, allowing for aggregation and persistence between calls.
     * @param {boolean} [opts.recursive=true] - Whether to recursively collect errors from child controls within a form group or array. Defaults to true.
     * @return {FormErrors} The collected errors as a key-value object where keys represent control names or paths and values represent their respective errors. Returns `undefined` if the provided control is valid and has no errors.
     */
    static getFormErrors: typeof getFormErrors;
    /**
     * Sets form errors on an AbstractControl, including nested controls within form groups or arrays.
     *
     * @param {AbstractControl} control - The form control, form group, or form array to apply the errors to.
     * @param {FormErrors} errors - An object representing the errors to be set on the control or its children.
     * @return {void} Does not return a value.
     */
    static setFormErrors: typeof setFormErrors;
}
export declare function isControlHasInput(controls: {
    [key: string]: AbstractControl;
}, controlName: string): boolean;
export declare function setCalculatedValue(controls: {
    [key: string]: AbstractControl;
}, controlName: string, value: number | undefined): void;
export declare function resetCalculatedValue(controls: {
    [key: string]: AbstractControl;
}, controlName: string): void;
