import { Observable } from 'rxjs';
import { Apollo, ExtraSubscriptionOptions, QueryRef } from 'apollo-angular';
import { ApolloCache, ApolloClient, FetchPolicy, MutationUpdaterFunction, OperationVariables, TypePolicies, Unmasked, WatchQueryFetchPolicy } from '@apollo/client/core';
import { ServiceError } from '../services/errors';
import { InjectionToken } from '@angular/core';
import { NetworkService } from '../services/network.service';
import { EmptyObject, TrackableMutationContext } from './graphql.utils';
import { Platform } from '@ionic/angular';
import { IEntity } from '../services/model/entity.model';
import { Resolvers } from '@apollo/client/core/types';
import { HttpLink } from 'apollo-angular/http';
import { ErrorPolicy, MutationBaseOptions } from '@apollo/client/core/watchQueryOptions';
import { Cache } from '@apollo/client/cache/core/types/Cache';
import { Environment } from '../../../environments/environment.class';
import { PropertyMap } from '../../shared/types';
import { StartableService } from '../../shared/services/startable-service.class';
import { StorageService } from '../../shared/storage/storage.service';
import { DocumentNode } from 'graphql';
import * as i0 from "@angular/core";
export interface WatchQueryOptions<V> {
    query: any;
    variables?: V;
    error?: ServiceError;
    fetchPolicy?: WatchQueryFetchPolicy;
}
export interface MutateQueryOptions<TData, TVariables = OperationVariables, TContext = TrackableMutationContext, TCache extends ApolloCache<any> = ApolloCache<any>> extends MutationBaseOptions<TData, TVariables, TContext, TCache> {
    mutation: any;
    variables?: TVariables;
    error?: ServiceError;
    context?: TContext;
    optimisticResponse?: Unmasked<NoInfer<TData>>;
    offlineResponse?: TData | ((context: TContext) => Promise<TData>);
    update?: MutationUpdaterFunction<TData, TVariables, TContext, TCache>;
    forceOffline?: boolean;
}
export declare const APP_GRAPHQL_TYPE_POLICIES: InjectionToken<TypePolicies>;
export declare const APP_GRAPHQL_FRAGMENTS: InjectionToken<DocumentNode[]>;
export interface ConnectionParams extends Record<string, string> {
    authToken?: string;
    authBasic?: string;
}
export declare class GraphqlService extends StartableService<ApolloClient<any>> {
    private platform;
    private apollo;
    private httpLink;
    private network;
    private storage;
    protected environment: Environment;
    private typePolicies;
    private fragments;
    private readonly _networkStatusChanged$;
    private httpParams;
    private wsParams;
    private connectionParams;
    private readonly _defaultFetchPolicy;
    private onNetworkError;
    private customErrors;
    private onResetAuth;
    get client(): ApolloClient<any>;
    get cache(): ApolloCache<any>;
    get defaultFetchPolicy(): WatchQueryFetchPolicy;
    constructor(platform: Platform, apollo: Apollo, httpLink: HttpLink, network: NetworkService, storage: StorageService, environment: Environment, typePolicies: TypePolicies, fragments: DocumentNode[]);
    setAuthToken(token: string): Promise<void>;
    setAuthBasic(basic: string): Promise<void>;
    /**
     * Allow to add a field resolver
     *  (see doc: https://www.apollographql.com/docs/react/data/local-state/#handling-client-fields-with-resolvers)
     *
     * @param resolvers
     */
    addResolver(resolvers: Resolvers | Resolvers[]): Promise<void>;
    query<T, V = EmptyObject>(opts: {
        query: any;
        variables?: V;
        error?: ServiceError;
        fetchPolicy?: FetchPolicy;
    }): Promise<T>;
    watchQueryRef<T, V = EmptyObject>(opts: WatchQueryOptions<V>): QueryRef<T, V>;
    queryRefValuesChanges<T, V = EmptyObject>(queryRef: QueryRef<T, V>, opts: WatchQueryOptions<V>): Observable<T>;
    watchQuery<T, V = EmptyObject>(opts: WatchQueryOptions<V>): Observable<T>;
    mutate<T, V = EmptyObject>(opts: MutateQueryOptions<T, V>): Promise<T>;
    subscribe<T, V = EmptyObject>(opts: {
        query: any;
        variables: V;
        fetchPolicy?: FetchPolicy;
        errorPolicy?: ErrorPolicy;
        error?: ServiceError;
    }, extra?: ExtraSubscriptionOptions): Observable<T>;
    insertIntoQueryCache<T, V = EmptyObject>(cache: ApolloCache<any>, opts: Cache.ReadQueryOptions<V, any> & {
        arrayFieldName: string;
        totalFieldName?: string;
        data: T;
        sortFn?: (d1: T, d2: T) => number;
        size?: number;
    }): void;
    addManyToQueryCache<T = any, V = EmptyObject>(cache: ApolloCache<any>, opts: Cache.ReadQueryOptions<V, any> & {
        arrayFieldName: string;
        totalFieldName?: string;
        data: T[];
        equalsFn?: (d1: T, d2: T) => boolean;
        sortFn?: (d1: T, d2: T) => number;
    }): number;
    /**
     * Remove from cache, and return if removed or not
     *
     * @param cache
     * @param opts
     */
    removeFromCachedQueryById<V = EmptyObject, ID = number>(cache: ApolloCache<any>, opts: Cache.ReadQueryOptions<V, any> & {
        arrayFieldName: string;
        totalFieldName?: string;
        ids: ID;
    }): boolean;
    /**
     * Remove ids from cache, and return the number of items removed
     *
     * @param cache
     * @param opts
     */
    removeFromCachedQueryByIds<V = EmptyObject, ID = number>(cache: ApolloCache<any>, opts: Cache.ReadQueryOptions<V, any> & {
        arrayFieldName: string;
        totalFieldName?: string;
        ids: ID[];
    }): number;
    updateToQueryCache<T extends IEntity<any>, V = EmptyObject>(cache: ApolloCache<any>, opts: Cache.ReadQueryOptions<V, any> & {
        arrayFieldName: string;
        totalFieldName?: string;
        data: T;
        equalsFn?: (d1: T, d2: T) => boolean;
    }): void;
    clearCache(client?: ApolloClient<any>): Promise<void>;
    registerCustomError(error: PropertyMap): void;
    protected ngOnStart(): Promise<ApolloClient<any>>;
    protected ngOnStop(): Promise<void>;
    protected resetClient(client?: ApolloClient<any>): Promise<void>;
    private toApolloError;
    private createAppErrorByCode;
    private getI18nErrorMessageByCode;
    private toAppError;
    static ɵfac: i0.ɵɵFactoryDeclaration<GraphqlService, [null, null, null, null, null, null, { optional: true; }, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<GraphqlService>;
}
