import { InjectionToken } from '@angular/core';
import { Keypair } from './crypto.service';
import { Department } from './model/department.model';
import { Account, UserSettings } from './model/account.model';
import { Person, UserProfileLabel } from './model/person.model';
import { LocalSettings, UsageMode } from './model/settings.model';
import { BehaviorSubject, Observable, Subject, Subscription } from 'rxjs';
import { FetchPolicy } from '@apollo/client/core';
import { Storage } from '@ionic/storage-angular';
import { BaseGraphqlService } from './base-graphql-service.class';
import { GraphqlService } from '../graphql/graphql.service';
import { LocalSettingsService } from './local-settings.service';
import { FormFieldDefinition, FormFieldDefinitionMap } from '../../shared/form/field.model';
import { NetworkService } from './network.service';
import { AuthTokenType } from './network.types';
import { FileService } from '../../shared/file/file.service';
import { Referential } from './model/referential.model';
import { Environment } from '../../../environments/environment.class';
import { ShowToastOptions } from '../../shared/toast/toasts';
import { OverlayEventDetail } from '@ionic/core';
import { ToastController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { TokenScope, UserToken } from './model/token.model';
import * as i0 from "@angular/core";
export declare interface AccountDetails {
    loaded: boolean;
    keypair: Keypair;
    authToken: string;
    authBasic?: string;
    pubkey: string;
    person: Person;
    department: Department;
    mainProfile: string;
}
export interface AuthData {
    username: string;
    password: string;
    offline?: boolean;
}
export interface RegisterData extends AuthData {
    account: Account;
}
export interface ChangePasswordData {
    password: string;
    confirmPassword: string;
    email: string;
}
export interface UserSettingsOptions {
    options: FormFieldDefinitionMap;
    remoteLocalSettingsKeys?: (keyof LocalSettings | string)[];
}
export declare const APP_USER_SETTINGS_OPTIONS: InjectionToken<UserSettingsOptions>;
export declare const APP_USER_TOKEN_SCOPES: InjectionToken<TokenScope[]>;
export declare interface IsEmailExistsVariables {
    email: string;
    hash: string;
}
interface IAuthChallenge {
    pubkey: string;
    challenge: string;
    signature: string;
}
export interface AccountWatchOptions {
    intervalInSeconds?: number;
}
export declare class AccountService extends BaseGraphqlService<Account, any, number, Account> {
    protected network: NetworkService;
    protected graphql: GraphqlService;
    protected settings: LocalSettingsService;
    protected storage: Storage;
    protected file: FileService;
    private translate;
    private toastController;
    protected environment: Environment;
    onLogin: Subject<Account>;
    onWillLogout: Subject<Account>;
    onLogout: Subject<void>;
    onChange: Subject<Account>;
    onAuthTokenChange: BehaviorSubject<string>;
    onAuthBasicChange: BehaviorSubject<string>;
    private _listenChangesSubscription;
    private readonly _enableListenChanges;
    private readonly _listenIntervalInSeconds;
    private _cache;
    private readonly _optionDefs;
    private readonly _remoteLocalSettingsKeys;
    private _$additionalFields;
    private _tokenType$;
    private _apiTokenEnabled;
    get account(): Account;
    get person(): Person;
    get department(): Department;
    get tokenType(): AuthTokenType;
    set tokenType(value: AuthTokenType);
    get apiTokenEnabled(): boolean;
    set apiTokenEnabled(value: boolean);
    constructor(network: NetworkService, graphql: GraphqlService, settings: LocalSettingsService, storage: Storage, file: FileService, translate: TranslateService, toastController: ToastController, environment: Environment, options: UserSettingsOptions);
    ngOnStart(): Promise<Account>;
    protected ngOnStop(): Promise<void>;
    isLogin(): boolean;
    isAuth(): boolean;
    hasMinProfile(userProfile: UserProfileLabel): boolean;
    hasExactProfile(label: UserProfileLabel): boolean;
    hasProfileAndIsEnable(userProfile: UserProfileLabel): boolean;
    isAdmin(): boolean;
    isSupervisor(): boolean;
    isUser(): boolean;
    /**
     * @deprecated
     * @param mode
     */
    isUsageMode(mode: UsageMode): boolean;
    isOnlyGuest(): boolean;
    /**
     *
     * @param recorderDepartment
     * @deprecated use ProgramRefService.canUserWriteEntity() instead
     */
    canUserWriteDataForDepartment(recorderDepartment: Referential | any): boolean;
    register(data: RegisterData): Promise<Account>;
    authenticate(data?: AuthData): Promise<void>;
    login(data: AuthData): Promise<Account>;
    reload(opts?: {
        showToast?: boolean;
    }): Promise<Account>;
    /**
     * Create or update an user account, to the remote storage
     *
     * @param account
     */
    save(account: Account): Promise<Account>;
    logout(): Promise<void>;
    /**
     * Load a account
     *
     * @param opts
     */
    load(opts?: {
        offline?: boolean;
        fetchPolicy?: FetchPolicy;
        query?: any;
    }): Promise<Account | undefined>;
    generateToken(flags: number): Promise<string | undefined>;
    /**
     * Check if email is available for new account registration.
     * Throw an error if not available
     *
     * @param email
     */
    checkEmailAvailable(email: string): Promise<void>;
    /**
     * Check if email is exists in server.
     *
     * @param email
     */
    isEmailExists(email: string): Promise<boolean>;
    sendConfirmationEmail(email: string, locale?: string): Promise<boolean>;
    confirmEmail(email: string, code: string): Promise<boolean>;
    sendResetPasswordEmail(username: string, locale?: string): Promise<boolean>;
    updatePubkey(data: AuthData): Promise<boolean>;
    resetPubkey(data: AuthData, token: string): Promise<boolean>;
    listenChanges(opts?: AccountWatchOptions): Subscription;
    get additionalFields(): FormFieldDefinition[];
    get $additionalFields(): Observable<FormFieldDefinition[]>;
    getAdditionalField(key: string): FormFieldDefinition | undefined;
    registerAdditionalField(field: FormFieldDefinition): void;
    get optionDefs(): FormFieldDefinition[];
    registerOption(def: FormFieldDefinition): void;
    registerOptions(defs: FormFieldDefinition[]): void;
    protected resetData(): void;
    protected loadData(opts?: {
        offline?: boolean;
        fetchPolicy?: FetchPolicy;
    }): Promise<Account>;
    protected listenSettings(): Promise<void>;
    protected restoreLocally(): Promise<Account | undefined>;
    /**
     * Save account into the local storage
     */
    protected saveLocally(): Promise<void>;
    saveLocalSettingsRemotely(source: LocalSettings): Promise<void>;
    /**
     * Create or update an user account
     *
     * @param account
     * @param keyPair
     */
    protected saveRemotely(account: Account, keyPair?: Keypair): Promise<Account>;
    /**
     * Create or update user settings
     *
     * @param settings
     */
    protected saveSettingsRemotely(settings: UserSettings): Promise<void>;
    protected authenticateAndGetToken(token?: string, counter?: number): Promise<string>;
    protected getAuthChallenge(): Promise<IAuthChallenge>;
    protected startListenRemoteChanges(opts?: AccountWatchOptions): Subscription;
    protected stopListenRemoteChanges(): void;
    protected watchChanges(opts?: AccountWatchOptions): Observable<Account>;
    protected copyIdAndUpdateDate(source: Account | undefined, target: Account): void;
    protected copyIdAndUpdateDateSettings(source: UserSettings | undefined, target: UserSettings): void;
    protected copyIdAndUpdateDateToken(sources: UserToken[] | undefined, targets: UserToken[]): void;
    protected showToast<T = any>(opts: ShowToastOptions): Promise<OverlayEventDetail<T>>;
    static ɵfac: i0.ɵɵFactoryDeclaration<AccountService, [null, null, null, null, null, { optional: true; }, { optional: true; }, null, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<AccountService>;
}
export {};
