import { Moment } from 'moment';
import { getPropertyByPath } from '../../../shared/functions';
import { FilterFn, KeyValueType, ObjectMap, ObjectMapEntry } from '../../../shared/types';
import { StoreObject } from '@apollo/client/core';
import { TreeItemEntityUtils } from './tree-item-entity.model';
import { SortDirection } from '@angular/material/sort';
import { AppPropertiesUtils } from '../../form/properties/properties.utils';
export declare interface Cloneable<T> {
    clone(): T;
}
export declare function entityToString(obj: Entity<any> | any, properties?: string[]): string | undefined;
export interface EntityAsObjectOptions {
    minify?: boolean;
    keepTypename?: boolean;
    keepLocalId?: boolean;
}
export interface IEntity<T, ID = number, AO extends EntityAsObjectOptions = EntityAsObjectOptions, FO = any> extends Cloneable<T> {
    id: ID;
    updateDate: Moment;
    __typename: string;
    equals(other: T): boolean;
    clone(opts?: AO & FO): T;
    copy(target: T, opts?: AO & FO): void;
    asObject(opts?: AO): any;
    fromObject(source: any, opts?: FO): void;
}
export declare type OmitFunctions<T extends IEntity<T, any>> = {
    [K in keyof T as T[K] extends Function ? never : K]: T[K];
};
export declare abstract class Entity<T extends Entity<T, ID, AO, FO>, ID = number, AO extends EntityAsObjectOptions = EntityAsObjectOptions, FO = any> implements IEntity<T, ID, AO, FO> {
    static TYPENAME: string;
    id: ID;
    updateDate: Moment;
    __typename: string;
    constructor(__typename?: string);
    clone(opts?: AO & FO): T;
    copy(target: T, opts?: AO & FO): void;
    asObject(opts?: AO): StoreObject;
    fromObject(source: any, opts?: FO): void;
    equals(other: T): boolean;
}
/**
 * @deprecated use standard instanceof instead
 * @param obj
 * @param constructor
 */
export declare function isInstanceOf<T>(obj: any, constructor: new (...args: any[]) => T): obj is T;
export declare abstract class EntityUtils {
    static isEntity<T extends IEntity<any>>(obj: T | any): obj is T;
    static isNotEmpty<T extends IEntity<any> | any>(obj: T, checkedAttribute?: keyof T): boolean;
    static isEmpty<T extends IEntity<any> | any>(obj: T, checkedAttribute?: keyof T): boolean;
    static equals<T extends IEntity<any> | any>(o1: T, o2: T, checkAttribute?: keyof T): boolean;
    static getPropertyByPath: typeof getPropertyByPath;
    static getArrayAsMap<T = any>(source?: ObjectMapEntry<T>[]): ObjectMap<T>;
    static getMapAsArray<T = any>(source?: ObjectMap<T>): ObjectMapEntry<T>[];
    static getPropertyArrayAsObject: typeof AppPropertiesUtils.arrayAsObject;
    static copyIdAndUpdateDate<T extends Entity<any, any, any>>(source: T, target: T): void;
    static fillLocalIds<T extends IEntity<T, any>>(items: T[], sequenceFactory: (firstEntity: T, incrementSize: number) => Promise<number>): Promise<void>;
    static cleanIdAndUpdateDate<T extends IEntity<T, any>>(source: T, defaultValue?: any): void;
    static cleanIdsAndUpdateDates<T extends IEntity<T, any>>(items: T[], defaultValue?: any): void;
    static sort<T extends IEntity<T, any> | any>(data: T[], sortBy?: keyof T | string, sortDirection?: SortDirection): T[];
    static sortComparator<T extends IEntity<T, any> | any>(sortBy?: keyof T | string, sortDirection?: SortDirection): (a: T, b: T) => number;
    static compare<T extends IEntity<T, any>>(value1: T, value2: T, direction: 1 | -1, checkAttribute?: keyof T): number;
    static compareValue<T>(value1: T, value2: T, direction: 1 | -1): number;
    static filter<T extends IEntity<T, any> | any>(data: T[], searchAttribute: string, searchText: string): T[];
    static searchTextFilter<T extends IEntity<T, any> | any>(searchAttribute: string | string[], searchText: string): FilterFn<T>;
    /**
     * Transform a batch entity tree into a array list. This method keep links parent/children.
     *
     * Method used to find batch without id (e.g. before local storage)
     *
     * @param source
     */
    static treeToArray: typeof TreeItemEntityUtils.treeToArray;
    static listOfTreeToArray: typeof TreeItemEntityUtils.listOfTreeToArray;
    /**
     * Fill parent attribute, of all children found in the tree
     *
     * @param source
     */
    static treeFillParent: typeof TreeItemEntityUtils.treeFillParent;
    static isLocal(entity: IEntity<any, any> | Entity<any, any>): boolean;
    static isRemote(entity: IEntity<any, any> | Entity<any, any>): boolean;
    static isLocalId(id: number): boolean;
    static isRemoteId(id: number): boolean;
    static getId<T extends IEntity<T, ID> = IEntity<any, any>, ID = any>(entity: T): ID;
    static hasId<T extends IEntity<any, any> = IEntity<any, any>>(entity: T): boolean;
    static hasNilId<T extends IEntity<any, any> = IEntity<any, any>>(entity: T): boolean;
    /**
     * Splits an array of entities into a key-value map using an entity ID as the key.
     *
     * @param {Array<T>} entities - The array of entities to be split by their ID.
     * @return {KeyValueType<T>} A key-value map with entity IDs as keys and entities as values.
     */
    static splitById<T extends IEntity<T, ID>, ID = any>(entities: T[]): KeyValueType<T>;
    /**
     * Collects the IDs from an array of entities and returns them as an array.
     *
     * @param {T[]} entities - An array of entities from which to collect IDs.
     * @return {ID[]} An array of IDs collected from the entities.
     */
    static collectIds<T extends IEntity<T, ID>, ID = any>(entities: T[]): ID[];
    /**
     * @deprecated Use collectIds() instead - will be remove in next major release
     */
    static collectById<T extends IEntity<T, ID>, ID = any>(entities: T[]): ID[];
    static arrayDistinctFilterFn<E extends IEntity<any, any>>(item: E, index: number, values: E[]): boolean;
    static findById<E extends IEntity<any, ID> = IEntity<any, any>, ID = number>(items: E[], id: ID): E | undefined;
    static fromObject<E extends IEntity<any, any>>(source: any, dataType: new () => E): E;
    /**
     * Recursively converts any object, entity, or array to a plain JavaScript object.
     * This method handles entities by calling their asObject() method, processes arrays recursively,
     * and provides protection against circular references.
     *
     * @param {any} source - The source object, entity, array, or primitive value to convert
     * @param {any} [opts] - Optional configuration object passed to entity asObject() methods
     * @param {WeakSet} [visited=new WeakSet()] - Internal parameter for circular reference detection
     * @returns {any} A plain JavaScript object representation of the source
     *
     * @example
     * // Convert a simple entity
     * const plainObject = EntityUtils.asObject(myEntity);
     *
     * @example
     * // Convert an array of entities
     * const plainArray = EntityUtils.asObject([entity1, entity2]);
     *
     * @example
     * // Convert with options
     * const result = EntityUtils.asObject(myEntity, { minify: true });
     */
    static asObject(source: any, opts?: any, visited?: WeakSet<object>): any;
}
