export declare abstract class UrlUtils {
    /**
     * Determines if a given URL is a relative URL.
     *
     * @param {string | null} href - The URL to check.
     * @return {boolean} True if the URL is relative, false otherwise.
     */
    static isRelativeUrl(href: string | null): boolean;
    /**
     * Determines if a given URL is an internal URL.
     *
     * @param {string | null} href - The URL to be checked. Can be a string representing the URL or null.
     * @return {boolean} Returns true if the URL is internal, otherwise false.
     */
    static isInternalUrl(href: string | null): boolean;
    static isExternalUrl(href: string | null): boolean;
    /**
     * Removes the query string from the given URL, returning only the base URL.
     *
     * @param {string} url - The URL from which the query string will be stripped.
     * @return {string} The URL without the query string.
     */
    static stripQuery(url: string): string;
    /**
     * Removes the fragment and query components from a given URL string.
     *
     * @param {string} url - The URL string to process.
     * @return {string} The URL without its fragment and query components.
     */
    static stripFragmentAndQuery(url: string): string;
    /**
     * Extracts the fragment identifier from the given URL.
     *
     * @param {string} url - The URL from which to extract the fragment.
     * @return {string} The fragment identifier of the URL, excluding the '#' symbol.
     */
    static getFragment(url: string): string;
    static resolveRelativeUrl(baseUrl: string, relativePath: string, opts?: {
        absoluteByDefault: boolean;
    }): string;
    /**
     * Resolves a given URL by normalizing its parts, removing unnecessary path segments like "." and "..".
     *
     * @param {string} url - The URL to be resolved. Must be a non-empty string.
     * @return {string} The resolved and cleaned URL string.
     * @throws {Error} If the provided URL is missing or undefined.
     */
    static normalizeUrl(url: string): string;
    /**
     * This function completes a base URL with an object containing key-value pairs
     * representing the query parameters to add. It properly encodes the
     * parameters and ensures they are added with the appropriate separator ('?' or '&').
     * Parameters with null or undefined values are automatically filtered out.
     *
     * @example
     * // In a template
     * <a [href]="'https://example.com' | appendQueryParams:{id: 123, name: 'test'}">Link</a>
     * // Result: https://example.com?id=123&name=test
     *
     * @example
     * // If the URL already contains parameters
     * <a [href]="'https://example.com?page=1' | appendQueryParams:{sort: 'asc'}">Link</a>
     * // Result: https://example.com?page=1&sort=asc
     *
     * @param url
     * @param queryParams
     */
    static appendQueryParams(url: string | any[], queryParams: {
        [key: string]: any;
    }): string;
    /**
     * Extracts and returns the root URL from the given complete URL string.
     *
     * @param {string} url - The complete URL string from which the root URL will be extracted.
     * @return {string} The root URL including the protocol and host (and port if present).
     * @throws {Error} Throws an error if the provided URL is missing or is in an invalid format.
     */
    static getRootUrl(url: string): string;
    /**
     * Retrieves the parent URL or directory path from the provided URL.
     * This method handles both relative and absolute URLs.
     *
     * @param {string} url - The URL from which to derive the parent path.
     *                       Must be a valid URL or relative path.
     * @return {string} The parent URL or directory path. Throws an error if the input is invalid.
     */
    static getParent(url: string): string;
    /**
     * Cleans the provided URL by encoding it safely and replacing specific patterns.
     *
     * @param {string} href - The URL string to be cleaned.
     * @return {string|null} The cleaned and encoded URL string, or null if an error occurs.
     */
    static cleanUrl(href: string): string;
    static addTrailingSlash(url: string): string;
    static addStartingSlash(url: string): string;
    static removeStartingSlash(url: string): string;
    static concat(basePath: string, relativePath: string): string;
}
