import { AbstractControl, UntypedFormArray, UntypedFormGroup, ValidationErrors } from '@angular/forms';
import { WaitForOptions } from './observables';
import { Predicate } from '@angular/core';
export interface FormErrors {
    [key: string]: ValidationErrors;
}
/**
 * Fill a form using a source entity
 *
 * @param target
 * @param entity
 * @param opts
 */
export declare function copyEntity2Form(entity: any, target: UntypedFormGroup, opts?: {
    emitEvent?: boolean;
    onlySelf?: boolean;
}): void;
export declare function getFormValueFromEntity(entity: any | undefined, form: UntypedFormGroup): {
    [key: string]: any;
};
/**
 * Transform an entity into a simple object, compatible with the given form
 *
 * @param source an entity (or subentity)
 * @param control
 * @param path
 */
export declare function adaptValueToControl(source: any | undefined, control: AbstractControl, path?: string): any;
export declare function logFormErrors(control: AbstractControl, logPrefix?: string, path?: string): void;
/**
 * Retrieves all validation errors from a given form control, including its child controls if it is a form group or array.
 *
 * @param {AbstractControl} control - The form control to retrieve errors from. This can be a form group, a form array, or a simple form control.
 * @param {Object} [opts] - Optional configurations for error retrieval.
 * @param {string} [opts.controlName] - The name or path of the control being processed, used for structuring error keys in the result.
 * @param {FormErrors} [opts.result] - The object to which errors will be added, allowing for aggregation and persistence between calls.
 * @param {boolean} [opts.recursive=true] - Whether to recursively collect errors from child controls within a form group or array. Defaults to true.
 * @return {FormErrors} The collected errors as a key-value object where keys represent control names or paths and values represent their respective errors. Returns `undefined` if the provided control is valid and has no errors.
 */
export declare function getFormErrors(control: AbstractControl, opts?: {
    controlName?: string;
    result?: FormErrors;
    recursive?: boolean;
}): FormErrors;
/**
 * Retrieves a control from the provided form/control structure using a dot-separated path.
 *
 * @param {AbstractControl} form - The root form group, form array, or control to start searching from.
 * @param {string} path - The dot-separated path specifying the control. For example, 'groupName.controlName' or 'arrayName.0.controlName'.
 * @param {boolean} [strict=true] - If true, throws an error if the full path cannot be resolved; if false, returns the last valid control encountered.
 * @return {AbstractControl} - The control at the specified path or the closest valid control if strict mode is disabled.
 * @throws {Error} - Throws an error if the path cannot be resolved and strict mode is enabled.
 */
export declare function getControlFromPath(form: AbstractControl, path: string, strict?: boolean): AbstractControl;
export declare function disableControls(form: UntypedFormGroup, paths: string[], opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function disableAndClearControls(form: UntypedFormGroup, paths: string[], opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function enableControls(form: UntypedFormGroup, paths: string[], opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function setControlsEnabled(form: UntypedFormGroup, enabled: boolean, paths: string[], opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
/**
 * Sets form errors on an AbstractControl, including nested controls within form groups or arrays.
 *
 * @param {AbstractControl} control - The form control, form group, or form array to apply the errors to.
 * @param {FormErrors} errors - An object representing the errors to be set on the control or its children.
 * @return {void} Does not return a value.
 */
export declare function setFormErrors(control: AbstractControl, errors: FormErrors): void;
export declare function enableControl(control: AbstractControl, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function disableControl(control: AbstractControl, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function disableAndClearControl(control: AbstractControl, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function setControlEnabled(control: AbstractControl, enabled: boolean, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
    required?: boolean;
}): void;
export declare function addValueInArray(arrayControl: UntypedFormArray, createControl: (value?: any) => AbstractControl, equals: (v1: any, v2: any) => boolean, isEmpty: (value: any) => boolean, value: any, options?: {
    emitEvent: boolean;
    insertAt?: number;
    allowManyNullValues?: boolean;
    allowDuplicateValue?: boolean;
}): boolean;
/**
 * Set an array using given default values. Each default value will be pass to the 'createControl()' function
 *
 * @param arrayControl
 * @param createControl
 * @param defaultValues
 * @param options
 */
export declare function initArrayControlsFromValues(arrayControl: UntypedFormArray, createControl: (value?: any) => AbstractControl, defaultValues: any[], options?: {
    emitEvent?: boolean;
}): boolean;
export declare function resizeArray(arrayControl: UntypedFormArray, createControl: () => AbstractControl, length: number, options?: {
    emitEvent?: boolean;
}): boolean;
export declare function removeValueInArray(arrayControl: UntypedFormArray, isEmpty: (value: any) => boolean, index: number): boolean;
export declare function clearValueInArray(arrayControl: UntypedFormArray, isEmpty: (value: any) => boolean, index: number): boolean;
export declare function markAllAsTouched(control: AbstractControl, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
}): void;
export declare function markFormGroupAsTouched(form: UntypedFormGroup, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
}): void;
export declare function markControlAsTouched(control: AbstractControl, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
}): void;
export declare function updateValueAndValidity(form: UntypedFormGroup, opts?: {
    onlySelf?: boolean;
    emitEvent?: boolean;
}): void;
export declare function markAsUntouched(form: UntypedFormGroup, opts?: {
    onlySelf?: boolean;
}): void;
/**
 * Wait end of async validation (not pending). This need to implement {pending: boolean}
 * return false
 */
export declare function waitWhilePending<T extends {
    pending: boolean;
}>(form: T, opts?: WaitForOptions): Promise<void>;
/**
 * Wait form is idle (not loading). This need to implement {loading: boolean}
 *
 * @param form
 * @param opts
 */
export declare function waitIdle<T extends {
    loading: boolean;
}>(form: T, opts?: WaitForOptions): Promise<void>;
export declare function filterFormErrorsByPrefix(errors: FormErrors, ...prefixes: string[]): FormErrors;
export declare function filterFormErrorsByPath(errors: FormErrors, filter: Predicate<string>): FormErrors;
export declare function filterFormErrors(errors: FormErrors, filter: Predicate<[string, any]>): FormErrors;
