import { ToastController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { StorageService } from '../storage/storage.service';
import * as i0 from "@angular/core";
export interface PrintOptions {
    id?: string | number;
    title?: string;
    showToast?: boolean;
    toastMessage?: string;
    style?: string;
}
export declare const PRINT_LOADING_STORAGE_KEY_PREFIX = "print-loading-";
export declare const PRINT_ID_QUERY_PARAM = "print-id";
export declare class PrintService {
    protected toastController: ToastController;
    protected translate: TranslateService;
    protected storage: StorageService;
    protected document: Document;
    private _printId;
    constructor(toastController: ToastController, translate: TranslateService, storage: StorageService, document: Document);
    nextJobId(): number;
    /**
     * Extracts and returns the job ID from a given URL.
     *
     * @param {string} [url] - The URL from which to extract the job ID. If not provided, `window.location.href` will be used.
     * @return {number | undefined} The extracted job ID as a number, or `undefined` if the job ID is not present or invalid.
     */
    getJobId(url?: string): string | undefined;
    /**
     * Determines whether the provided URL or the current page's URL contains
     * a specific query parameter, indicating it's a printing-related URL.
     *
     * @param {string} [url] - The optional URL to check. If not provided, the current page's URL is used.
     * @param {string} [queryParam='print-id'] - The query parameter to look for, defaulting to 'print-id'.
     * @return {boolean} Returns true if the query parameter is found in the URL; otherwise, false.
     */
    isPrintingUrl(url?: string, queryParam?: string): boolean;
    printElement(element: HTMLElement, opts?: PrintOptions): Promise<number | string>;
    printHTML(html: string, opts?: PrintOptions): Promise<number | string>;
    /**
     * Prints the content of a URL by rendering it in an iframe.
     *
     * @param {string} url The URL of the content to print. This parameter is required.
     * @param {Omit<PrintOptions, 'style'>} [opts] Optional settings for printing, excluding the 'style' option.
     * @return {Promise<number>} A promise that resolves to the ID of the print job.
     */
    printUrl(url: string, opts?: Omit<PrintOptions, 'style'>): Promise<number | string>;
    /**
     * Prints the content of the specified iframe element.
     *
     * @param {HTMLIFrameElement} iframe - The iframe element whose content needs to be printed. Must be a valid and loaded iframe element.
     * @param {PrintOptions} [opts] - Optional parameters for the print operation, which may include options for toast messages and job configuration.
     * @return {Promise<number>} A promise that resolves to the unique ID of the print job.
     * @throws Will throw an error if the required 'iframe' argument is missing or invalid.
     */
    printIframe(iframe: HTMLIFrameElement, opts?: PrintOptions): Promise<number | string>;
    markAsLoading(id: string | number): Promise<void>;
    markAsLoaded(id: string | number): Promise<void>;
    isLoaded(id: string | number): Promise<boolean>;
    isLoading(id: string | number): Promise<boolean>;
    protected createPrintIframeByUrl(url: string, opts?: {
        id?: number | string;
    }): HTMLIFrameElement;
    protected createPrintIframeByHtml(html: string, opts?: {
        title?: string;
        inlineStyle?: string;
    }): HTMLIFrameElement;
    /**
     * Applies the styles from the provided document, including external stylesheets and inline styles,
     * to the specified iframe's document.
     *
     * @param {HTMLIFrameElement} iframe - The iframe to which the styles are applied.
     * @return {void} Does not return a value.
     */
    protected applyDocumentStyleToIframe(iframe: HTMLIFrameElement): void;
    protected waitIdle(iframe: HTMLIFrameElement, opts?: {
        id?: number | string;
        timeout?: number;
    }): Promise<void>;
    /**
     * Triggers the print functionality of the given iframe and resolves or rejects based on the print completion or timeout.
     *
     * @param {HTMLIFrameElement} iframe - The iframe element to be printed.
     * @param {{timeout?: number}} [opts] - An optional configuration object.
     * @param {number} [opts.timeout] - The maximum time in milliseconds to wait for the print to complete before rejecting the promise.
     * @return {Promise<void>} A promise that resolves after the print is completed or rejects if a timeout occurs.
     */
    protected print(iframe: HTMLIFrameElement, opts?: {
        timeout?: number;
    }): Promise<void>;
    private showToast;
    static ɵfac: i0.ɵɵFactoryDeclaration<PrintService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<PrintService>;
}
