import { AbstractControl, AsyncValidatorFn, UntypedFormControl, ValidationErrors, ValidatorFn } from '@angular/forms';
import { Moment, unitOfTime } from 'moment';
import { Observable, Subscription } from 'rxjs';
export declare class SharedValidators {
    private static _DOUBLE_REGEXP_CACHE;
    private static getDoubleRegexp;
    static equals(otherControlName: string): ValidatorFn;
    static readonly I18N_ERROR_KEYS: {
        required: string;
        min: string;
        max: string;
        minlength: string;
        maxlength: string;
        pubkey: string;
        validDate: string;
        dateIsAfter: string;
        dateIsBefore: string;
        dateRange: string;
        dateMinDuration: string;
        dateMaxDuration: string;
        maxDecimals: string;
        decimal: string;
        integer: string;
        precision: string;
        email: string;
        latitude: string;
        longitude: string;
        pattern: string;
        unique: string;
        entity: string;
        invalid: string;
        inconsistent: string;
        equals: string;
    };
    static latitude(control: UntypedFormControl): ValidationErrors | null;
    static longitude(control: UntypedFormControl): ValidationErrors | null;
    static object(control: UntypedFormControl): ValidationErrors | null;
    static entity(control: UntypedFormControl): ValidationErrors | null;
    static entities(control: UntypedFormControl): ValidationErrors | null;
    static empty(control: UntypedFormControl): ValidationErrors | null;
    static pubkey(control: UntypedFormControl): ValidationErrors | null;
    static integer(control: UntypedFormControl): ValidationErrors | null;
    static decimal(opts?: {
        maxDecimals?: number;
    }): ValidatorFn;
    /**
     * Check precision. e.g. if precision=0.5 then value=0.6 is invalid, but 1.5 is valid
     *
     * @param precision
     */
    static precision(precision: number): ValidatorFn;
    static validDateTime(requiredTime?: boolean): ValidatorFn;
    static validDate: ValidatorFn;
    static validDateAllowNoTime: ValidatorFn;
    static dateIsAfter(previousValue: Moment, errorParam: string, granularity?: unitOfTime.StartOf): ValidatorFn;
    static dateIsBefore(maxValue: Moment, errorParam: string, granularity?: unitOfTime.StartOf): ValidatorFn;
    static dateRangeEnd(startDateFieldName: string, msg?: string): ValidatorFn;
    static dateRangeStart(endDateFieldName: string, msg?: string): ValidatorFn;
    static copyParentErrors(errorNames?: string[]): ValidatorFn;
    static clearError(control: AbstractControl, errorCode: string): void;
}
export declare class SharedFormGroupValidators {
    static dateRange(startDateField: string, endDateField: string, opts?: {
        msg?: string;
        fieldOnly?: boolean;
        skipIfNoTime?: boolean;
    } | string): ValidatorFn;
    static dateMaxDuration(startDateField: string, endDateField: string, maxDuration: number, durationUnit?: moment.unitOfTime.Diff, opts?: {
        skipIfNoTime?: boolean;
    }): ValidatorFn;
    static dateMinDuration(startDateField: string, endDateField: string, minDuration: number, durationUnit?: moment.unitOfTime.Diff): ValidatorFn;
    static requiredIf(fieldName: string, anotherFieldToCheck: string | AbstractControl, opts?: {
        fieldOnly?: boolean;
        isEmpty?: (control: AbstractControl) => boolean;
        predicate?: (control: AbstractControl) => boolean;
    }): ValidatorFn;
    static requiredIfTrue(fieldName: string, anotherFieldToCheck: string | AbstractControl, opts?: {
        fieldOnly?: boolean;
        isEmpty?: (control: AbstractControl) => boolean;
    }): ValidatorFn;
    static requiredIfEmpty(fieldName: string, anotherFieldToCheck: string, opts?: {
        fieldOnly?: boolean;
        isEmpty?: (control: AbstractControl) => boolean;
    }): ValidatorFn;
    static propagateIfDirty(fieldName: string, fieldNameToPropagate: string, valueToPropagate: any): ValidatorFn;
    /**
     * Same as compose, but keep only the first errors (instead of the union)
     *
     * @param validators
     */
    first(validators: (ValidatorFn | null | undefined)[]): ValidationErrors | null;
}
export declare class SharedFormArrayValidators {
    /**
     * Validate uniqueness of an entity in a FormArray
     *
     * @param controlName the name of the control in FormArray
     */
    static uniqueEntity(controlName: string): ValidatorFn;
    static requiredArrayMinLength(minLength?: number): ValidatorFn;
    static arrayMaxLength(maxLength?: number): ValidatorFn;
    /**
     * Validate the sum of control values in an FormArray not overflow the max value
     *
     * @param controlName the name of the control in FormArray
     * @param max the maximum value
     */
    static validSumMaxValue(controlName: string, max: number): ValidatorFn;
}
export declare type ObservableValidatorFn = (control: AbstractControl) => Observable<ValidationErrors | null>;
export declare type PromiseValidatorFn = (control: AbstractControl) => Promise<ValidationErrors | null>;
export declare class SharedAsyncValidators {
    private static DEBOUNCE_TIME_VALIDATOR_ID;
    /**
     * Add a debounce time to a validator.
     *
     * @param form
     * @param validatorFn
     * @param opts Use opts.stopSubject to stop the validator, before end
     */
    static debounceTime(validatorFn: ValidatorFn | AsyncValidatorFn, opts?: {
        dispose?: Observable<any>;
        debounceTime?: number;
        markForCheck?: () => void;
        debug?: boolean;
    }): ObservableValidatorFn;
    /**
     * Add an async validator, that can be disposed by the returned subscription
     *
     * @param form
     * @param validatorFn the validator or job to execute
     * @param opts
     */
    static registerAsyncValidator(form: AbstractControl, validatorFn: ValidatorFn | AsyncValidatorFn, opts?: {
        debounceTime?: number;
        markForCheck?: () => void;
        debug?: boolean;
    }): Subscription;
}
